﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace Sage200ThreadLogon
{
    /// <summary>
    /// The blocking queue. For a full discussion see:
    /// http://stackoverflow.com/questions/530211/creating-a-blocking-queuet-in-net/530228#530228
    /// </summary>
    /// <typeparam name="T"></typeparam>
    class BlockingQueue<T>
    {
        private readonly Queue<T> queue = new Queue<T>();
        private readonly int _maxSize;
        
        /// <summary>
        /// 
        /// </summary>
        /// <param name="maxSize">The maximum number of elements the queue accepts before it blocks.</param>
        public BlockingQueue(int maxSize) 
        { 
            _maxSize = maxSize; 
        }

        /// <summary>
        /// Enqueues an item to the queue.
        /// </summary>
        /// <param name="item">The item.</param>
        public void Enqueue(T item)
        {
            lock (queue)
            {
                //If the queue is full then wait until it
                //reduces in size.
                while (queue.Count >= _maxSize)
                {
                    Monitor.Wait(queue);
                }
                queue.Enqueue(item);
                if (queue.Count == 1)
                {
                    // wake up any blocked dequeue 
                    Monitor.PulseAll(queue);
                }
            }
        }

        /// <summary>
        /// Dequeues an item from the queue. The calling 
        /// thread will block until there's an item in the queue.
        /// </summary>
        /// <returns>An item.</returns>
        public T Dequeue()
        {
            lock (queue)
            {
                //If the queue is empty wait until there is an item
                //before continuing.
                while (queue.Count == 0)
                {
                    Monitor.Wait(queue);
                }
                T item = queue.Dequeue();
                if (queue.Count == _maxSize - 1)
                {
                    // wake up any blocked enqueue 
                    Monitor.PulseAll(queue);
                }
                return item;
            }
        }

        public T Peek()
        {
            lock (queue)
            {
                return queue.Peek();
            }
        }
    }
}
