﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Sage.Accounting.SOP;

namespace Sage200ThreadLogon
{
    class Connector
    {
        private const string SOP_DEFAULT_PICKLIST_LAYOUT = @"Reports\SOP\Picking List.report";

        private static object _syncLock = new object();

        int _id;
        Sage.Accounting.Application _application = new Sage.Accounting.Application();

        public Connector(int id)
        {
            _id = id;
        }

        public void StartThread(object info)
        {
            //Print an arbitrary three times.
            for (int j = 0; j < 3; j++)
            {
                //_syncLock is a static object declared at class level.
                lock (_syncLock)
                {
                    //Authenticate to Sage200
                    _application.Connect("admin", "admin");
                    System.Diagnostics.Debug.WriteLine(System.Threading.Thread.CurrentThread.ManagedThreadId + " - Connected");

                    //Set the active company.
                    for (int i = 0; i < _application.Companies.Count; i++)
                    {
                        if (_application.Companies[i].Name == "Sage200_2010_DemoData")
                        {
                            //When the ActiveCompany property is set Sage sets the 
                            //current thread's CurrentPrincipal property with logon
                            //context.
                            _application.ActiveCompany = _application.Companies[i];
                            break;
                        }
                    }
                }

                //Print an arbitary picklist; change picklist and printer name accordingly.
                PrintPickList("0000006028", false, true, string.Empty, string.Empty, "HP Deskjet 6980 series");

                _application.Disconnect();
                System.Diagnostics.Debug.WriteLine(System.Threading.Thread.CurrentThread.ManagedThreadId + " - Disconnected");
            }
        }

        /// <summary>
        /// Prints a single picklist using by specifying an order.
        /// </summary>
        /// <param name="orderReturnNo">The order/return number.</param>
        /// <param name="includeCompleted">Whether to print completed orders.</param>
        /// <param name="printPickList">True/false whether to print the picklist.</param>
        /// <param name="printFileName">True/false whether to export the picklist.</param>
        /// <param name="pickListNoteLayout">The picking list layout; when null or empty will print the default layout.</param>
        /// <param name="printerName">The printer name to print; if null or empty the report will print to the default printer.</param>
        public void PrintPickList(string orderReturnNo, bool includeCompleted, bool printPickList, string printFileName, string pickListNoteLayout, string printerName)
        {
            string result = string.Empty;

            SOPPrintPickingListCoordinator coordinator = new SOPPrintPickingListCoordinator();

            //SuppressReprint Exception
            Sage.Accounting.Application.AllowableWarnings.Add(coordinator, typeof(Sage.Accounting.Exceptions.Ex20246Exception));

            Dictionary<string, object> reportCriteria = new Dictionary<string, object>();

            coordinator.ClearCriteria();
            coordinator.PrintOnlyPrevUnprinted = includeCompleted;
            coordinator.OrderNumberFrom = orderReturnNo;
            coordinator.OrderNumberTo = orderReturnNo;

            ReportUtility.SetCriteria(reportCriteria, Sage.Accounting.Common.ReportingConstants.CRITERIA_SOPOrderReturn_OrderDocumentNumber, orderReturnNo);

            // If selected to only print previously unprinted items  
            if (coordinator.PrintOnlyPrevUnprinted == true)
            {
                // Add PickingListPrintStatusID Criterion 
                ReportUtility.SetCriteria(reportCriteria, Sage.Accounting.Common.ReportingConstants.CRITERIA_SOPAllocationLine_PickingListPrintStatusID, 0);
            }

            if (!coordinator.PickingListItems.IsEmpty)
            {
                string layout = string.IsNullOrEmpty(pickListNoteLayout) ? SOP_DEFAULT_PICKLIST_LAYOUT : pickListNoteLayout;

                //Create a new ReportRequest and queue it to be printed.
                ReportQueueManager.RunReport(new ReportRequest()
                {
                    ReportFileName = layout,
                    Criteria = reportCriteria,
                    ExportFilePath = printFileName,
                    Print = printPickList,
                    PrinterName = printerName
                });
                coordinator.UpdatePrintStatus();
            }
        }
    }
}
