﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Security.Principal;

namespace Sage200ThreadLogon
{
    class ReportRequest
    {
        public ReportRequest()
        {
            //Capture the Principal of the calling thread
            Principal = Thread.CurrentPrincipal;
            //Used for debug and tracing purposes.
            ThreadId = Thread.CurrentThread.ManagedThreadId;
        }

        /// <summary>
        /// Gets the thread id of the thread that created the object.
        /// </summary>
        public int ThreadId { get; private set; }

        /// <summary>
        /// The report file.
        /// </summary>
        public string ReportFileName { get; set; }

        /// <summary>
        /// The criteria/parameters for the report.
        /// </summary>
        public Dictionary<string, object> Criteria { get; set; }
        
        /// <summary>
        /// A string indicating the export path of the report. Leave null or empty not to export.
        /// </summary>
        public string ExportFilePath { get; set; }

        /// <summary>
        /// True/false to indicate whether the report is to be printed.
        /// </summary>
        public bool Print { get; set; }

        /// <summary>
        /// The printer name of the locally installed printer.
        /// </summary>
        public string PrinterName { get; set; }

        /// <summary>
        /// Gets the principal of the thread issuing the print request.
        /// </summary>
        public IPrincipal Principal { get; private set; }
    }
}
